export default class Codes {
  private codes: Function[]

  constructor() {
    this.codes = []
  }

  get length(): number {
    // Array.length doesn't count string key.
    return Object.keys(this.codes).length
  }

  size(): number {
    return this.codes.length
  }

  push(code: Function): this
  push(title: string, code: Function): this
  push(title: any, code?: any): this {
    if (title instanceof Function) {
      code = title
      title = ''
    }

    if (title === '') {
      this.codes.push(code)
    } else {
      this.codes[title] = code
    }
    return this
  }

  runAt(index: number | string, args: any[] = []): any {
    const code = this.codes[index]
    return code(...args)
  }

  runFirst(args: any[] = []): any {
    return this.runAt(0, args)
  }

  runLast(args: any[] = []): any {
    return this.runAt(this.codes.length - 1, args)
  }

  runMatch(pattern: RegExp, args: any[] = []): any {
    for (let title in this.codes) {
      if (title.match(pattern)) {
        return this.codes[title](...args)
      }
    }
    throw new Error(`Not matched to: ${pattern}`)
  }
}
